@@ -17,6 +17,8 @@ core.features = {
 	area_store_persistent_ids = true,
 	pathfinder_works = true,
 	object_step_has_moveresult = true,
+	direct_velocity_on_players = true,
+	use_texture_alpha_string_modes = true,
 }
 
 function core.has_feature(arg)
