@@ -31,12 +31,43 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include <algorithm>
 #include "client/renderingengine.h"
 
+// struct MeshBufListList
+void MeshBufListList::clear()
+{
+	for (auto &list : lists)
+		list.clear();
+}
+
+void MeshBufListList::add(scene::IMeshBuffer *buf, v3s16 position, u8 layer)
+{
+	// Append to the correct layer
+	std::vector<MeshBufList> &list = lists[layer];
+	const video::SMaterial &m = buf->getMaterial();
+	for (MeshBufList &l : list) {
+		// comparing a full material is quite expensive so we don't do it if
+		// not even first texture is equal
+		if (l.m.TextureLayer[0].Texture != m.TextureLayer[0].Texture)
+			continue;
+
+		if (l.m == m) {
+			l.bufs.emplace_back(position, buf);
+			return;
+		}
+	}
+	MeshBufList l;
+	l.m = m;
+	l.bufs.emplace_back(position, buf);
+	list.emplace_back(l);
+}
+
+// ClientMap
+
 ClientMap::ClientMap(
 		Client *client,
 		MapDrawControl &control,
 		s32 id
 ):
-	Map(dout_client, client),
+	Map(client),
 	scene::ISceneNode(RenderingEngine::get_scene_manager()->getRootSceneNode(),
 		RenderingEngine::get_scene_manager(), id),
 	m_client(client),
@@ -122,20 +153,20 @@ void ClientMap::updateDrawList()
 	}
 	m_drawlist.clear();
 
-	v3f camera_position = m_camera_position;
-	v3f camera_direction = m_camera_direction;
-	f32 camera_fov = m_camera_fov;
+	const v3f camera_position = m_camera_position;
+	const v3f camera_direction = m_camera_direction;
 
 	// Use a higher fov to accomodate faster camera movements.
 	// Blocks are cropped better when they are drawn.
-	// Or maybe they aren't? Well whatever.
-	camera_fov *= 1.2;
+	const f32 camera_fov = m_camera_fov * 1.1f;
 
 	v3s16 cam_pos_nodes = floatToInt(camera_position, BS);
 	v3s16 p_blocks_min;
 	v3s16 p_blocks_max;
 	getBlocksInViewRange(cam_pos_nodes, &p_blocks_min, &p_blocks_max);
 
+	// Number of blocks currently loaded by the client
+	u32 blocks_loaded = 0;
 	// Number of blocks with mesh in rendering range
 	u32 blocks_in_range_with_mesh = 0;
 	// Number of blocks occlusion culled
@@ -160,6 +191,7 @@ void ClientMap::updateDrawList()
 		MapSector *sector = sector_it.second;
 		v2s16 sp = sector->getPos();
 
+		blocks_loaded += sector->size();
 		if (!m_control.range_all) {
 			if (sp.X < p_blocks_min.X || sp.X > p_blocks_max.X ||
 					sp.Y < p_blocks_min.Z || sp.Y > p_blocks_max.Z)
@@ -181,8 +213,10 @@ void ClientMap::updateDrawList()
 				if not seen on display
 			*/
 
-			if (block->mesh)
-				block->mesh->updateCameraOffset(m_camera_offset);
+			if (!block->mesh) {
+				// Ignore if mesh doesn't exist
+				continue;
+			}
 
 			float range = 100000 * BS;
 			if (!m_control.range_all)
@@ -193,13 +227,6 @@ void ClientMap::updateDrawList()
 					camera_direction, camera_fov, range, &d))
 				continue;
 
-
-			/*
-				Ignore if mesh doesn't exist
-			*/
-			if (!block->mesh)
-				continue;
-
 			blocks_in_range_with_mesh++;
 
 			/*
@@ -228,52 +255,9 @@ void ClientMap::updateDrawList()
 	g_profiler->avg("MapBlock meshes in range [#]", blocks_in_range_with_mesh);
 	g_profiler->avg("MapBlocks occlusion culled [#]", blocks_occlusion_culled);
 	g_profiler->avg("MapBlocks drawn [#]", m_drawlist.size());
+	g_profiler->avg("MapBlocks loaded [#]", blocks_loaded);
 }
 
-struct MeshBufList
-{
-	video::SMaterial m;
-	std::vector<scene::IMeshBuffer*> bufs;
-};
-
-struct MeshBufListList
-{
-	/*!
-	 * Stores the mesh buffers of the world.
-	 * The array index is the material's layer.
-	 * The vector part groups vertices by material.
-	 */
-	std::vector<MeshBufList> lists[MAX_TILE_LAYERS];
-
-	void clear()
-	{
-		for (auto &list : lists)
-			list.clear();
-	}
-
-	void add(scene::IMeshBuffer *buf, u8 layer)
-	{
-		// Append to the correct layer
-		std::vector<MeshBufList> &list = lists[layer];
-		const video::SMaterial &m = buf->getMaterial();
-		for (MeshBufList &l : list) {
-			// comparing a full material is quite expensive so we don't do it if
-			// not even first texture is equal
-			if (l.m.TextureLayer[0].Texture != m.TextureLayer[0].Texture)
-				continue;
-
-			if (l.m == m) {
-				l.bufs.push_back(buf);
-				return;
-			}
-		}
-		MeshBufList l;
-		l.m = m;
-		l.bufs.push_back(buf);
-		list.push_back(l);
-	}
-};
-
 void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 {
 	bool is_transparent_pass = pass == scene::ESNRP_TRANSPARENT;
@@ -293,19 +277,20 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 	/*
 		Get animation parameters
 	*/
-	float animation_time = m_client->getAnimationTime();
-	int crack = m_client->getCrackLevel();
-	u32 daynight_ratio = m_client->getEnv().getDayNightRatio();
+	const float animation_time = m_client->getAnimationTime();
+	const int crack = m_client->getCrackLevel();
+	const u32 daynight_ratio = m_client->getEnv().getDayNightRatio();
 
-	v3f camera_position = m_camera_position;
-	v3f camera_direction = m_camera_direction;
-	f32 camera_fov = m_camera_fov;
+	const v3f camera_position = m_camera_position;
+	const v3f camera_direction = m_camera_direction;
+	const f32 camera_fov = m_camera_fov;
 
 	/*
 		Get all blocks and draw all visible ones
 	*/
 
 	u32 vertex_count = 0;
+	u32 drawcall_count = 0;
 
 	// For limiting number of mesh animations per frame
 	u32 mesh_animate_count = 0;
@@ -318,6 +303,7 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 	MeshBufListList drawbufs;
 
 	for (auto &i : m_drawlist) {
+		v3s16 block_pos = i.first;
 		MapBlock *block = i.second;
 
 		// If the mesh of the block happened to get deleted, ignore it
@@ -383,7 +369,7 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 						material.setFlag(video::EMF_WIREFRAME,
 							m_control.show_wireframe);
 
-						drawbufs.add(buf, layer);
+						drawbufs.add(buf, block_pos, layer);
 					}
 				}
 			}
@@ -392,6 +378,9 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 
 	TimeTaker draw("Drawing mesh buffers");
 
+	core::matrix4 m; // Model matrix
+	v3f offset = intToFloat(m_camera_offset, BS);
+
 	// Render all layers in order
 	for (auto &lists : drawbufs.lists) {
 		for (MeshBufList &list : lists) {
@@ -403,7 +392,14 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 			}
 			driver->setMaterial(list.m);
 
-			for (scene::IMeshBuffer *buf : list.bufs) {
+			drawcall_count += list.bufs.size();
+			for (auto &pair : list.bufs) {
+				scene::IMeshBuffer *buf = pair.second;
+
+				v3f block_wpos = intToFloat(pair.first * MAP_BLOCKSIZE, BS);
+				m.setTranslation(block_wpos - offset);
+
+				driver->setTransform(video::ETS_WORLD, m);
 				driver->drawMeshBuffer(buf);
 				vertex_count += buf->getVertexCount();
 			}
@@ -417,6 +413,7 @@ void ClientMap::renderMap(video::IVideoDriver* driver, s32 pass)
 	}
 
 	g_profiler->avg(prefix + "vertices drawn [#]", vertex_count);
+	g_profiler->avg(prefix + "drawcalls [#]", drawcall_count);
 }
 
 static bool getVisibleBrightness(Map *map, const v3f &p0, v3f dir, float step,
@@ -608,5 +605,3 @@ void ClientMap::PrintInfo(std::ostream &out)
 {
 	out<<"ClientMap: ";
 }
-
-
