@@ -45,12 +45,16 @@ public:
 
 	video::SColor crosshair_argb;
 	video::SColor selectionbox_argb;
+
 	bool use_crosshair_image = false;
+	bool use_object_crosshair_image = false;
 	std::string hotbar_image = "";
 	bool use_hotbar_image = false;
 	std::string hotbar_selected_image = "";
 	bool use_hotbar_selected_image = false;
 
+	bool pointing_at_object = false;
+
 	Hud(gui::IGUIEnvironment *guienv, Client *client, LocalPlayer *player,
 			Inventory *inventory);
 	~Hud();
@@ -77,6 +81,8 @@ public:
 		m_selected_face_normal = face_normal;
 	}
 
+	bool hasElementOfType(HudElementType type);
+
 	void drawLuaElements(const v3s16 &camera_offset);
 
 private:
@@ -91,6 +97,12 @@ private:
 
 	void drawItem(const ItemStack &item, const core::rect<s32> &rect, bool selected);
 
+	void drawCompassTranslate(HudElement *e, video::ITexture *texture,
+			const core::rect<s32> &rect, int way);
+
+	void drawCompassRotate(HudElement *e, video::ITexture *texture,
+			const core::rect<s32> &rect, int way);
+
 	float m_hud_scaling; // cached minetest setting
 	float m_scale_factor;
 	v3s16 m_camera_offset;
@@ -111,6 +123,8 @@ private:
 
 	video::SMaterial m_selection_material;
 
+	scene::SMeshBuffer m_rotation_mesh_buffer;
+
 	enum
 	{
 		HIGHLIGHT_BOX,
