@@ -248,7 +248,7 @@ bool GUIKeyChangeMenu::acceptInput()
 {
 	for (key_setting *k : key_settings) {
 		std::string default_key;
-		g_settings->getDefaultNoEx(k->setting_name, default_key);
+		Settings::getLayer(SL_DEFAULTS)->getNoEx(k->setting_name, default_key);
 
 		if (k->key.sym() != default_key)
 			g_settings->set(k->setting_name, k->key.sym());
@@ -360,7 +360,7 @@ bool GUIKeyChangeMenu::OnEvent(const SEvent& event)
 		{
 			if (!canTakeFocus(event.GUIEvent.Element))
 			{
-				dstream << "GUIMainMenu: Not allowing focus change."
+				infostream << "GUIKeyChangeMenu: Not allowing focus change."
 				<< std::endl;
 				// Returning true disables focus change
 				return true;
