@@ -204,6 +204,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 	PROTOCOL VERSION 39:
 		Updated set_sky packet
 		Adds new sun, moon and stars packets
+		Minimap modes
 */
 
 #define LATEST_PROTOCOL_VERSION 39
@@ -225,21 +226,8 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #define PASSWORD_SIZE 28       // Maximum password length. Allows for
                                // base64-encoded SHA-1 (27+\0).
 
-/*
-	Changes by FORMSPEC_API_VERSION:
-
-	FORMSPEC VERSION 1:
-		(too much)
-	FORMSPEC VERSION 2:
-		Forced real coordinates
-		background9[]: 9-slice scaling parameters
-	FORMSPEC VERSION 3:
-		Formspec elements are drawn in the order of definition
-		bgcolor[]: use 3 parameters (bgcolor, formspec (now an enum), fbgcolor)
-		box[] and image[] elements enable clipping by default
-		new element: scroll_container[]
-*/
-#define FORMSPEC_API_VERSION 3
+// See also: Formspec Version History in doc/lua_api.txt
+#define FORMSPEC_API_VERSION 4
 
 #define TEXTURENAME_ALLOWED_CHARS "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_.-"
 
@@ -762,7 +750,18 @@ enum ToClientCommand
 		u8[len] formspec
 	*/
 
-	TOCLIENT_NUM_MSG_TYPES = 0x62,
+	TOCLIENT_MINIMAP_MODES = 0x62,
+	/*
+		u16 count // modes
+		u16 mode  // wanted current mode index after change
+		for each mode
+			u16 type
+			std::string label
+			u16 size
+			std::string extra
+	*/
+
+	TOCLIENT_NUM_MSG_TYPES = 0x63,
 };
 
 enum ToServerCommand
@@ -1032,7 +1031,7 @@ const static std::string accessDeniedStrings[SERVER_ACCESSDENIED_MAX] = {
 	"This server has experienced an internal error. You will now be disconnected."
 };
 
-enum PlayerListModifer: u8
+enum PlayerListModifer : u8
 {
 	PLAYER_LIST_INIT,
 	PLAYER_LIST_ADD,
