@@ -24,6 +24,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include <iostream>
 #include <map>
 #include <vector>
+#include "util/Optional.h"
 
 struct ObjectProperties
 {
@@ -53,6 +54,7 @@ struct ObjectProperties
 	s8 glow = 0;
 	std::string nametag = "";
 	video::SColor nametag_color = video::SColor(255, 255, 255, 255);
+	Optional<video::SColor> nametag_bgcolor = nullopt;
 	f32 automatic_face_movement_max_rotation_per_sec = -1.0f;
 	std::string infotext;
 	//! For dropped items, this contains item information.
@@ -62,6 +64,7 @@ struct ObjectProperties
 	float zoom_fov = 0.0f;
 	bool use_texture_alpha = false;
 	bool shaded = true;
+	bool show_on_minimap = false;
 
 	ObjectProperties();
 	std::string dump();
