@@ -25,7 +25,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 #include "porting.h"
 
-#if defined(__FreeBSD__)  || defined(__NetBSD__) || defined(__DragonFly__)
+#if defined(__FreeBSD__)  || defined(__NetBSD__) || defined(__DragonFly__) || defined(__OpenBSD__)
 	#include <sys/types.h>
 	#include <sys/sysctl.h>
 	extern char **environ;
@@ -56,6 +56,10 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 	#include <crt_externs.h>
 #endif
 
+#if defined(__HAIKU__)
+        #include <FindDirectory.h>
+#endif
+
 #include "config.h"
 #include "debug.h"
 #include "filesys.h"
@@ -321,6 +325,12 @@ bool getCurrentExecPath(char *buf, size_t len)
 	return true;
 }
 
+#elif defined(__HAIKU__)
+
+bool getCurrentExecPath(char *buf, size_t len)
+{
+	return find_path(B_APP_IMAGE_SYMBOL, B_FIND_PATH_IMAGE_PATH, NULL, buf, len) == B_OK;
+}
 
 //// Solaris
 #elif defined(__sun) || defined(sun)
@@ -709,29 +719,48 @@ int mt_snprintf(char *buf, const size_t buf_size, const char *fmt, ...)
 	return c;
 }
 
-bool openURL(const std::string &url)
+static bool open_uri(const std::string &uri)
 {
-	if ((url.substr(0, 7) != "http://" && url.substr(0, 8) != "https://") ||
-			url.find_first_of("\r\n") != std::string::npos) {
-		errorstream << "Invalid url: " << url << std::endl;
+	if (uri.find_first_of("\r\n") != std::string::npos) {
+		errorstream << "Unable to open URI as it is invalid, contains new line: " << uri << std::endl;
 		return false;
 	}
 
 #if defined(_WIN32)
-	return (intptr_t)ShellExecuteA(NULL, NULL, url.c_str(), NULL, NULL, SW_SHOWNORMAL) > 32;
+	return (intptr_t)ShellExecuteA(NULL, NULL, uri.c_str(), NULL, NULL, SW_SHOWNORMAL) > 32;
 #elif defined(__ANDROID__)
-	openURLAndroid(url);
+	openURIAndroid(uri);
 	return true;
 #elif defined(__APPLE__)
-	const char *argv[] = {"open", url.c_str(), NULL};
+	const char *argv[] = {"open", uri.c_str(), NULL};
 	return posix_spawnp(NULL, "open", NULL, NULL, (char**)argv,
 		(*_NSGetEnviron())) == 0;
 #else
-	const char *argv[] = {"xdg-open", url.c_str(), NULL};
+	const char *argv[] = {"xdg-open", uri.c_str(), NULL};
 	return posix_spawnp(NULL, "xdg-open", NULL, NULL, (char**)argv, environ) == 0;
 #endif
 }
 
+bool open_url(const std::string &url)
+{
+	if (url.substr(0, 7) != "http://" && url.substr(0, 8) != "https://") {
+		errorstream << "Unable to open browser as URL is missing schema: " << url << std::endl;
+		return false;
+	}
+
+	return open_uri(url);
+}
+
+bool open_directory(const std::string &path)
+{
+	if (!fs::IsDir(path)) {
+		errorstream << "Unable to open directory as it does not exist: " << path << std::endl;
+		return false;
+	}
+
+	return open_uri(path);
+}
+
 // Load performance counter frequency only once at startup
 #ifdef _WIN32
 
