@@ -63,9 +63,12 @@ struct EnumString;
 struct NoiseParams;
 class Schematic;
 class ServerActiveObject;
+struct collisionMoveResult;
 
+extern struct EnumString es_TileAnimationType[];
 
-ContentFeatures    read_content_features     (lua_State *L, int index);
+void               read_content_features     (lua_State *L, ContentFeatures &f,
+                                              int index);
 void               push_content_features     (lua_State *L,
                                               const ContentFeatures &c);
 
@@ -196,4 +199,4 @@ void               push_hud_element          (lua_State *L, HudElement *elem);
 
 HudElementStat     read_hud_change           (lua_State *L, HudElement *elem, void **value);
 
-extern struct EnumString es_TileAnimationType[];
+void               push_collision_move_result(lua_State *L, const collisionMoveResult &res);
