@@ -17,8 +17,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef C_TYPES_H_
-#define C_TYPES_H_
+#pragma once
 
 extern "C" {
 #include "lua.h"
@@ -60,5 +59,3 @@ public:
 
 
 extern EnumString es_ItemType[];
-
-#endif /* C_TYPES_H_ */
