@@ -72,6 +72,9 @@ private:
 	// get_description(self)
 	static int l_get_description(lua_State *L);
 
+	// get_short_description(self)
+	static int l_get_short_description(lua_State *L);
+
 	// clear(self) -> true
 	static int l_clear(lua_State *L);
 
