@@ -18,8 +18,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef L_SOUND_H_
-#define L_SOUND_H_
+#pragma once
 
 #include "lua_api/l_base.h"
 
@@ -32,5 +31,3 @@ private:
 public:
 	static void Initialize(lua_State *L, int top);
 };
-
-#endif
