@@ -239,15 +239,6 @@ int ModApiUtil::l_is_yes(lua_State *L)
 	return 1;
 }
 
-// is_nan(arg)
-int ModApiUtil::l_is_nan(lua_State *L)
-{
-	NO_MAP_LOCK_REQUIRED;
-
-	lua_pushboolean(L, isNaN(L, 1));
-	return 1;
-}
-
 // get_builtin_path()
 int ModApiUtil::l_get_builtin_path(lua_State *L)
 {
@@ -259,6 +250,17 @@ int ModApiUtil::l_get_builtin_path(lua_State *L)
 	return 1;
 }
 
+// get_user_path()
+int ModApiUtil::l_get_user_path(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+
+	std::string path = porting::path_user;
+	lua_pushstring(L, path.c_str());
+
+	return 1;
+}
+
 // compress(data, method, level)
 int ModApiUtil::l_compress(lua_State *L)
 {
@@ -493,9 +495,9 @@ void ModApiUtil::Initialize(lua_State *L, int top)
 	API_FCT(get_password_hash);
 
 	API_FCT(is_yes);
-	API_FCT(is_nan);
 
 	API_FCT(get_builtin_path);
+	API_FCT(get_user_path);
 
 	API_FCT(compress);
 	API_FCT(decompress);
@@ -526,7 +528,6 @@ void ModApiUtil::InitializeClient(lua_State *L, int top)
 	API_FCT(write_json);
 
 	API_FCT(is_yes);
-	API_FCT(is_nan);
 
 	API_FCT(compress);
 	API_FCT(decompress);
@@ -550,6 +551,7 @@ void ModApiUtil::InitializeAsync(lua_State *L, int top)
 	API_FCT(is_yes);
 
 	API_FCT(get_builtin_path);
+	API_FCT(get_user_path);
 
 	API_FCT(compress);
 	API_FCT(decompress);
