@@ -55,9 +55,6 @@ ClientScripting::ClientScripting(Client *client):
 	InitializeModApi(L, top);
 	lua_pop(L, 1);
 
-	if (client->getMinimap())
-		LuaMinimap::create(L, client->getMinimap());
-
 	// Push builtin initialization type
 	lua_pushstring(L, "client");
 	lua_setglobal(L, "INIT");
@@ -69,6 +66,7 @@ void ClientScripting::InitializeModApi(lua_State *L, int top)
 {
 	LuaItemStack::Register(L);
 	ItemStackMetaRef::Register(L);
+	LuaRaycast::Register(L);
 	StorageRef::Register(L);
 	LuaMinimap::Register(L);
 	NodeMetaRef::RegisterClient(L);
@@ -93,3 +91,8 @@ void ClientScripting::on_camera_ready(Camera *camera)
 {
 	LuaCamera::create(getStack(), camera);
 }
+
+void ClientScripting::on_minimap_ready(Minimap *minimap)
+{
+	LuaMinimap::create(getStack(), minimap);
+}
