@@ -62,6 +62,10 @@ ServerScripting::ServerScripting(Server* server):
 
 	if (g_settings->getBool("secure.enable_security")) {
 		initializeSecurity();
+	} else {
+		warningstream << "\\!/ Mod security should never be disabled, as it allows any mod to "
+				<< "access the host machine."
+				<< "Mods should use minetest.request_insecure_environment() instead \\!/" << std::endl;
 	}
 
 	lua_getglobal(L, "core");
@@ -121,8 +125,3 @@ void ServerScripting::InitializeModApi(lua_State *L, int top)
 	ModApiStorage::Initialize(L, top);
 	ModApiChannels::Initialize(L, top);
 }
-
-void log_deprecated(const std::string &message)
-{
-	log_deprecated(NULL, message);
-}
