@@ -290,7 +290,7 @@ inline void writeS8(u8 *data, s8 i)
 
 inline void writeS16(u8 *data, s16 i)
 {
-	writeU16(data, (u16)i); 
+	writeU16(data, (u16)i);
 }
 
 inline void writeS32(u8 *data, s32 i)
@@ -440,22 +440,16 @@ MAKE_STREAM_WRITE_FXN(video::SColor, ARGB8, 4);
 ////
 
 // Creates a string with the length as the first two bytes
-std::string serializeString(const std::string &plain);
-
-// Creates a string with the length as the first two bytes from wide string
-std::string serializeWideString(const std::wstring &plain);
+std::string serializeString16(const std::string &plain);
 
 // Reads a string with the length as the first two bytes
-std::string deSerializeString(std::istream &is);
-
-// Reads a wide string with the length as the first two bytes
-std::wstring deSerializeWideString(std::istream &is);
+std::string deSerializeString16(std::istream &is);
 
 // Creates a string with the length as the first four bytes
-std::string serializeLongString(const std::string &plain);
+std::string serializeString32(const std::string &plain);
 
 // Reads a string with the length as the first four bytes
-std::string deSerializeLongString(std::istream &is);
+std::string deSerializeString32(std::istream &is);
 
 // Creates a string encoded in JSON format (almost equivalent to a C string literal)
 std::string serializeJsonString(const std::string &plain);
@@ -469,239 +463,3 @@ std::string serializeJsonStringIfNeeded(const std::string &s);
 
 // Parses a string serialized by serializeJsonStringIfNeeded.
 std::string deSerializeJsonStringIfNeeded(std::istream &is);
-
-// Creates a string consisting of the hexadecimal representation of `data`
-std::string serializeHexString(const std::string &data, bool insert_spaces=false);
-
-// Creates a string containing comma delimited values of a struct whose layout is
-// described by the parameter format
-bool serializeStructToString(std::string *out,
-	std::string format, void *value);
-
-// Reads a comma delimited string of values into a struct whose layout is
-// decribed by the parameter format
-bool deSerializeStringToStruct(std::string valstr,
-	std::string format, void *out, size_t olen);
-
-////
-//// BufReader
-////
-
-#define MAKE_BUFREADER_GETNOEX_FXN(T, N, S) \
-	inline bool get ## N ## NoEx(T *val)    \
-	{                                       \
-		if (pos + S > size)                 \
-			return false;                   \
-		*val = read ## N(data + pos);       \
-		pos += S;                           \
-		return true;                        \
-	}
-
-#define MAKE_BUFREADER_GET_FXN(T, N) \
-	inline T get ## N()              \
-	{                                \
-		T val;                       \
-		if (!get ## N ## NoEx(&val)) \
-			throw SerializationError("Attempted read past end of data"); \
-		return val;                  \
-	}
-
-class BufReader {
-public:
-	BufReader(const u8 *data_, size_t size_) :
-		data(data_),
-		size(size_)
-	{
-	}
-
-	MAKE_BUFREADER_GETNOEX_FXN(u8,    U8,       1);
-	MAKE_BUFREADER_GETNOEX_FXN(u16,   U16,      2);
-	MAKE_BUFREADER_GETNOEX_FXN(u32,   U32,      4);
-	MAKE_BUFREADER_GETNOEX_FXN(u64,   U64,      8);
-	MAKE_BUFREADER_GETNOEX_FXN(s8,    S8,       1);
-	MAKE_BUFREADER_GETNOEX_FXN(s16,   S16,      2);
-	MAKE_BUFREADER_GETNOEX_FXN(s32,   S32,      4);
-	MAKE_BUFREADER_GETNOEX_FXN(s64,   S64,      8);
-	MAKE_BUFREADER_GETNOEX_FXN(f32,   F1000,    4);
-	MAKE_BUFREADER_GETNOEX_FXN(v2s16, V2S16,    4);
-	MAKE_BUFREADER_GETNOEX_FXN(v3s16, V3S16,    6);
-	MAKE_BUFREADER_GETNOEX_FXN(v2s32, V2S32,    8);
-	MAKE_BUFREADER_GETNOEX_FXN(v3s32, V3S32,   12);
-	MAKE_BUFREADER_GETNOEX_FXN(v3f,   V3F1000, 12);
-	MAKE_BUFREADER_GETNOEX_FXN(video::SColor, ARGB8, 4);
-
-	bool getStringNoEx(std::string *val);
-	bool getWideStringNoEx(std::wstring *val);
-	bool getLongStringNoEx(std::string *val);
-	bool getRawDataNoEx(void *data, size_t len);
-
-	MAKE_BUFREADER_GET_FXN(u8,            U8);
-	MAKE_BUFREADER_GET_FXN(u16,           U16);
-	MAKE_BUFREADER_GET_FXN(u32,           U32);
-	MAKE_BUFREADER_GET_FXN(u64,           U64);
-	MAKE_BUFREADER_GET_FXN(s8,            S8);
-	MAKE_BUFREADER_GET_FXN(s16,           S16);
-	MAKE_BUFREADER_GET_FXN(s32,           S32);
-	MAKE_BUFREADER_GET_FXN(s64,           S64);
-	MAKE_BUFREADER_GET_FXN(f32,           F1000);
-	MAKE_BUFREADER_GET_FXN(v2s16,         V2S16);
-	MAKE_BUFREADER_GET_FXN(v3s16,         V3S16);
-	MAKE_BUFREADER_GET_FXN(v2s32,         V2S32);
-	MAKE_BUFREADER_GET_FXN(v3s32,         V3S32);
-	MAKE_BUFREADER_GET_FXN(v3f,           V3F1000);
-	MAKE_BUFREADER_GET_FXN(video::SColor, ARGB8);
-	MAKE_BUFREADER_GET_FXN(std::string,   String);
-	MAKE_BUFREADER_GET_FXN(std::wstring,  WideString);
-	MAKE_BUFREADER_GET_FXN(std::string,   LongString);
-
-	inline void getRawData(void *val, size_t len)
-	{
-		if (!getRawDataNoEx(val, len))
-			throw SerializationError("Attempted read past end of data");
-	}
-
-	inline size_t remaining()
-	{
-		assert(pos <= size);
-		return size - pos;
-	}
-
-	const u8 *data;
-	size_t size;
-	size_t pos = 0;
-};
-
-#undef MAKE_BUFREADER_GET_FXN
-#undef MAKE_BUFREADER_GETNOEX_FXN
-
-
-////
-//// Vector-based write routines
-////
-
-inline void putU8(std::vector<u8> *dest, u8 val)
-{
-	dest->push_back((val >> 0) & 0xFF);
-}
-
-inline void putU16(std::vector<u8> *dest, u16 val)
-{
-	dest->push_back((val >> 8) & 0xFF);
-	dest->push_back((val >> 0) & 0xFF);
-}
-
-inline void putU32(std::vector<u8> *dest, u32 val)
-{
-	dest->push_back((val >> 24) & 0xFF);
-	dest->push_back((val >> 16) & 0xFF);
-	dest->push_back((val >>  8) & 0xFF);
-	dest->push_back((val >>  0) & 0xFF);
-}
-
-inline void putU64(std::vector<u8> *dest, u64 val)
-{
-	dest->push_back((val >> 56) & 0xFF);
-	dest->push_back((val >> 48) & 0xFF);
-	dest->push_back((val >> 40) & 0xFF);
-	dest->push_back((val >> 32) & 0xFF);
-	dest->push_back((val >> 24) & 0xFF);
-	dest->push_back((val >> 16) & 0xFF);
-	dest->push_back((val >>  8) & 0xFF);
-	dest->push_back((val >>  0) & 0xFF);
-}
-
-inline void putS8(std::vector<u8> *dest, s8 val)
-{
-	putU8(dest, val);
-}
-
-inline void putS16(std::vector<u8> *dest, s16 val)
-{
-	putU16(dest, val);
-}
-
-inline void putS32(std::vector<u8> *dest, s32 val)
-{
-	putU32(dest, val);
-}
-
-inline void putS64(std::vector<u8> *dest, s64 val)
-{
-	putU64(dest, val);
-}
-
-inline void putF1000(std::vector<u8> *dest, f32 val)
-{
-	putS32(dest, val * FIXEDPOINT_FACTOR);
-}
-
-inline void putV2S16(std::vector<u8> *dest, v2s16 val)
-{
-	putS16(dest, val.X);
-	putS16(dest, val.Y);
-}
-
-inline void putV3S16(std::vector<u8> *dest, v3s16 val)
-{
-	putS16(dest, val.X);
-	putS16(dest, val.Y);
-	putS16(dest, val.Z);
-}
-
-inline void putV2S32(std::vector<u8> *dest, v2s32 val)
-{
-	putS32(dest, val.X);
-	putS32(dest, val.Y);
-}
-
-inline void putV3S32(std::vector<u8> *dest, v3s32 val)
-{
-	putS32(dest, val.X);
-	putS32(dest, val.Y);
-	putS32(dest, val.Z);
-}
-
-inline void putV3F1000(std::vector<u8> *dest, v3f val)
-{
-	putF1000(dest, val.X);
-	putF1000(dest, val.Y);
-	putF1000(dest, val.Z);
-}
-
-inline void putARGB8(std::vector<u8> *dest, video::SColor val)
-{
-	putU32(dest, val.color);
-}
-
-inline void putString(std::vector<u8> *dest, const std::string &val)
-{
-	if (val.size() > STRING_MAX_LEN)
-		throw SerializationError("String too long");
-
-	putU16(dest, val.size());
-	dest->insert(dest->end(), val.begin(), val.end());
-}
-
-inline void putWideString(std::vector<u8> *dest, const std::wstring &val)
-{
-	if (val.size() > WIDE_STRING_MAX_LEN)
-		throw SerializationError("String too long");
-
-	putU16(dest, val.size());
-	for (size_t i = 0; i != val.size(); i++)
-		putU16(dest, val[i]);
-}
-
-inline void putLongString(std::vector<u8> *dest, const std::string &val)
-{
-	if (val.size() > LONG_STRING_MAX_LEN)
-		throw SerializationError("String too long");
-
-	putU32(dest, val.size());
-	dest->insert(dest->end(), val.begin(), val.end());
-}
-
-inline void putRawData(std::vector<u8> *dest, const void *src, size_t len)
-{
-	dest->insert(dest->end(), (u8 *)src, (u8 *)src + len);
-}
