@@ -33,11 +33,12 @@ const char *g_version_hash = VERSION_GITHASH;
 const char *g_build_info =
 	"BUILD_TYPE=" BUILD_TYPE "\n"
 	"RUN_IN_PLACE=" STR(RUN_IN_PLACE) "\n"
+	"USE_CURL=" STR(USE_CURL) "\n"
+#ifndef SERVER
 	"USE_GETTEXT=" STR(USE_GETTEXT) "\n"
 	"USE_SOUND=" STR(USE_SOUND) "\n"
-	"USE_CURL=" STR(USE_CURL) "\n"
 	"USE_FREETYPE=" STR(USE_FREETYPE) "\n"
-	"USE_LUAJIT=" STR(USE_LUAJIT) "\n"
+#endif
 	"STATIC_SHAREDIR=" STR(STATIC_SHAREDIR)
 #if USE_GETTEXT && defined(STATIC_LOCALEDIR)
 	"\n" "STATIC_LOCALEDIR=" STR(STATIC_LOCALEDIR)
